@@ -1,6 +1,7 @@
 `getSplits` <-
 function(Symbol,from='1970-01-01',to=Sys.Date(),env=parent.frame(),src='yahoo',
-         auto.assign=FALSE,auto.update=FALSE,verbose=FALSE,...) {
+         auto.assign=FALSE,auto.update=FALSE,verbose=FALSE,...,
+         curl.options=list()) {
 
   # Function written by Joshua Ulrich, using
   # getSymbols.yahoo as a guide.
@@ -15,20 +16,22 @@ function(Symbol,from='1970-01-01',to=Sys.Date(),env=parent.frame(),src='yahoo',
   from.posix <- .dateToUNIX(from)
   to.posix <- .dateToUNIX(to)
 
-  tmp <- tempfile()
-  on.exit(unlink(tmp))
-
   handle <- .getHandle()
   yahoo.URL <- .yahooURL(Symbol.name, from.posix, to.posix,
                          "1d", "split", handle)
-  curl::curl_download(yahoo.URL, destfile=tmp, quiet=!verbose, handle=handle$ch)
 
-  fr <- read.csv(tmp, as.is=TRUE)
+  conn <- curl::curl(yahoo.URL, handle=handle$ch)
+  fr <- try(read.csv(conn, as.is=TRUE), silent=TRUE)
+
+  if (inherits(fr, "try-error")) {
+    fr <- retry.yahoo(Symbol.name, from.posix, to.posix, "1d", "split", conn)
+  }
 
   if(NROW(fr)==0) {
     fr <- NA
   } else {
-    fr$V3 <- vapply(parse(text=fr[,2]), eval, numeric(1))
+    fr[,2] <- gsub(":", "/", fr[,2], fixed = TRUE)
+    fr$V3 <- 1 / vapply(parse(text=fr[,2]), eval, numeric(1))
     fr <- xts(fr$V3, as.Date(fr[,1], "%Y-%m-%d"))
     colnames(fr) <- paste(Symbol.name,'spl',sep='.')
   }
