@@ -225,9 +225,6 @@ formals(loadSymbols) <- loadSymbols.formals
 
     # establish session
     new.session <- function() {
-      tmp <- tempfile()
-      on.exit(unlink(tmp))
-
       for (i in 1:5) {
         h <- curl::new_handle()
         curl::handle_setopt(h, .list = curl.options)
@@ -235,7 +232,7 @@ formals(loadSymbols) <- loadSymbols.formals
         # random query to avoid cache
         ru <- paste(sample(c(letters, 0:9), 4), collapse = "")
         cu <- paste0("https://finance.yahoo.com?", ru)
-        curl::curl_download(cu, tmp, handle = h)
+        z <- curl::curl_fetch_memory(cu, handle = h)
         if (NROW(curl::handle_cookies(h)) > 0)
           break;
         Sys.sleep(0.1)
@@ -307,9 +304,6 @@ function(Symbols,env,return.class='xts',index.class="Date",
 
      handle <- .getHandle(curl.options)
 
-     tmp <- tempfile()
-     on.exit(unlink(tmp))
-
      returnSym <- Symbols
      noDataSym <- NULL
      for(i in seq_along(Symbols)) {
@@ -340,30 +334,15 @@ function(Symbols,env,return.class='xts',index.class="Date",
 
        yahoo.URL <- .yahooURL(Symbols.name, from.posix, to.posix,
                               interval, "history", handle)
-       dl <- try(curl::curl_download(yahoo.URL, destfile = tmp,
-                                     quiet = !verbose, handle = handle$ch),
-                  silent = TRUE)
+       conn <- curl::curl(yahoo.URL, handle = handle$ch)
+       fr <- try(read.csv(conn, na.strings="null"), silent = TRUE)
 
-       if (inherits(dl, "try-error")) {
-         # warn user about the failure
-         warning(Symbols.name, " download failed; trying again.",
-                 call. = FALSE, immediate. = TRUE)
-         # re-create handle
-         handle <- .getHandle(curl.options, force.new = TRUE)
-         # try again
-         yahoo.URL <- .yahooURL(Symbols.name, from.posix, to.posix,
-                                interval, "history", handle)
-         dl <- try(curl::curl_download(yahoo.URL, destfile = tmp,
-                                       quiet = !verbose, handle = handle$ch),
-                    silent = TRUE)
-         # error if second attempt also failed
-         if (inherits(dl, "try-error")) {
-           stop(Symbols.name, " download failed after two attempts. Error",
-                " message:\n", attr(dl, "condition")$message, call. = FALSE)
-         }
+       if (inherits(fr, "try-error")) {
+         fr <- retry.yahoo(Symbols.name, from.posix, to.posix, interval,
+                           "history", curl.options = curl.options,
+                           na.strings = NULL)
        }
 
-       fr <- read.csv(tmp, na.strings="null")
        if(verbose) cat("done.\n")
        fr <- xts(as.matrix(fr[,-1]),
                  as.Date(fr[,1]),
@@ -391,7 +370,7 @@ function(Symbols,env,return.class='xts',index.class="Date",
 
        fr <- convert.time.series(fr=fr,return.class=return.class)
        if(is.xts(fr))
-         indexClass(fr) <- index.class
+         tclass(fr) <- index.class
 
        Symbols[[i]] <-toupper(gsub('\\^','',Symbols[[i]])) 
        if(auto.assign)
@@ -440,8 +419,8 @@ function(Symbols,env,return.class='xts',index.class="Date",
         if(!hasArg("verbose")) verbose <- FALSE
         if(!hasArg("auto.assign")) auto.assign <- TRUE
 
-        if(!requireNamespace("XML", quietly=TRUE))
-          stop("package:",dQuote("XML"),"cannot be loaded.")
+        if(!requireNamespace("xml2", quietly=TRUE))
+          stop("package:",dQuote("xml2"),"cannot be loaded.")
 
         yahoo.URL <- "https://info.finance.yahoo.co.jp/history/"
 
@@ -492,10 +471,8 @@ function(Symbols,env,return.class='xts',index.class="Date",
             
             page <- 1
             totalrows <- c()
-            tmp <- tempfile()
-            on.exit(unlink(tmp))
             while (TRUE) {
-                download.file(paste(yahoo.URL,
+                URL <- paste(yahoo.URL,
                                     "?code=",Symbols.name,
                                     "&sm=",from.m,
                                     "&sd=",sprintf('%.2d',from.d),
@@ -505,10 +482,10 @@ function(Symbols,env,return.class='xts',index.class="Date",
                                     "&ey=",to.y,
                                     "&tm=d",
                                     "&p=",page,
-                                    sep=''),destfile=tmp,quiet=!verbose)
+                                    sep='')
                 
-                fdoc <- XML::htmlParse(tmp)
-                rows <- XML::xpathApply(fdoc, "//table[@class='boardFin yjSt marB6']//tr")
+                fdoc <- xml2::read_html(URL)
+                rows <- xml2::xml_find_all(fdoc, "//table[@class='boardFin yjSt marB6']//tr")
                 if (length(rows) <= 1) break
                 
                 totalrows <- c(totalrows, rows)
@@ -524,19 +501,19 @@ function(Symbols,env,return.class='xts',index.class="Date",
             cols <- c('Open','High','Low','Close','Volume','Adjusted')
             
             firstrow <- totalrows[[1]]
-            cells <- XML::getNodeSet(firstrow, "th")
+            cells <- xml2::xml_find_all(firstrow, "th")
             if (length(cells) == 5) cols <- cols[-(5:6)]
 
             # Process from the start, for easier stocksplit management
             totalrows <- rev(totalrows)
             mat <- matrix(0, ncol=length(cols) + 1, nrow=0, byrow=TRUE)
             for(row in totalrows) {
-                cells <- XML::getNodeSet(row, "td")
+                cells <- xml2::xml_find_all(row, "td")
                 
                 # 2 cells means it is a stocksplit row
                 # So extract stocksplit data and recalculate the matrix we have so far
-                if (length(cells) == 2 & length(cols) == 6 & nrow(mat) > 1) {
-                    ss.data <- as.numeric(na.omit(as.numeric(unlist(strsplit(XML::xmlValue(cells[[2]]), "[^0-9]+")))))
+                if (length(cells) == 2 && length(cols) == 6 & nrow(mat) > 1) {
+                    ss.data <- as.numeric(na.omit(as.numeric(unlist(strsplit(xml2::xml_text(cells[[2]]), "[^0-9]+")))))
                     factor <- ss.data[2] / ss.data[1]
                     
                     mat <- rbind(t(apply(mat[-nrow(mat),], 1, function(x) {
@@ -550,10 +527,10 @@ function(Symbols,env,return.class='xts',index.class="Date",
                 # \u5e74 = "year"
                 # \u6708 = "month"
                 # \u65e5 = "day"
-                date <- as.Date(XML::xmlValue(cells[[1]]), format="%Y\u5e74%m\u6708%d\u65e5")
+                date <- as.Date(xml2::xml_text(cells[[1]]), format="%Y\u5e74%m\u6708%d\u65e5")
                 entry <- c(date)
                 for(n in 2:length(cells)) {
-                    entry <- cbind(entry, as.numeric(gsub(",", "", XML::xmlValue(cells[[n]]))))
+                    entry <- cbind(entry, as.numeric(gsub(",", "", xml2::xml_text(cells[[n]]))))
                 }
                 
                 mat <- rbind(mat, entry)
@@ -565,7 +542,7 @@ function(Symbols,env,return.class='xts',index.class="Date",
             
             fr <- convert.time.series(fr=fr,return.class=return.class)
             if(is.xts(fr))
-                indexClass(fr) <- index.class
+                tclass(fr) <- index.class
             
             Symbols[[i]] <- symname
             if(auto.assign)
@@ -771,9 +748,6 @@ function(Symbols,env,return.class='xts',
      if(!hasArg("auto.assign")) auto.assign <- TRUE
      FRED.URL <- "https://fred.stlouisfed.org/series"
 
-     tmp <- tempfile()
-     on.exit(unlink(tmp))
-
      returnSym <- Symbols
      noDataSym <- NULL
 
@@ -781,8 +755,8 @@ function(Symbols,env,return.class='xts',
        if(verbose) cat("downloading ",Symbols[[i]],".....\n\n")
        test <- try({
        URL <- paste(FRED.URL, "/", Symbols[[i]], "/downloaddata/", Symbols[[i]], ".csv", sep="")
-       try.download.file(URL, destfile=tmp, quiet=!verbose, ...)
-       fr <- read.csv(tmp,na.string=".")
+       fr <- read.csv(curl::curl(URL),na.string=".")
+
        if(verbose) cat("done.\n")
        fr <- xts(as.matrix(fr[,-1]),
                  as.Date(fr[,1],origin='1970-01-01'),
@@ -1216,9 +1190,6 @@ function(Symbols,env,return.class='xts',
      if(!hasArg("verbose")) verbose <- FALSE
      if(!hasArg("auto.assign")) auto.assign <- TRUE
 
-     tmp <- tempfile()
-     on.exit(unlink(tmp))
-
      returnSym <- Symbols
      noDataSym <- NULL
 
@@ -1336,9 +1307,6 @@ getSymbols.av <- function(Symbols, env, api.key,
          " cannot be loaded.", call.=FALSE)
   }
   
-  tmp <- tempfile()
-  on.exit(file.remove(tmp))
-  
   #
   # For daily, weekly, and monthly data, timestamps are "yyyy-mm-dd".
   # For intraday data, timestamps are "yyyy-mm-dd HH:MM:SS".
@@ -1382,10 +1350,8 @@ getSymbols.av <- function(Symbols, env, api.key,
                   "&datatype=", data.type,
                   "&apikey=", api.key)
     
-    download.file(url=URL, destfile=tmp, quiet=!verbose)
-
     if (data.type == "json") {
-      lst <- jsonlite::fromJSON(tmp)
+      lst <- jsonlite::fromJSON(URL)
 
       #
       # Errors return a list with one element: An error message
@@ -1458,7 +1424,7 @@ getSymbols.av <- function(Symbols, env, api.key,
       mat <- xts(rows, tm.stamps, src="alphavantage", updated=updated)
       mat <- convert.time.series(mat, return.class=return.class)
     } else {
-      mat <- as.xts(read.zoo(tmp, header=TRUE, sep=","),
+      mat <- as.xts(read.zoo(curl::curl(URL), header=TRUE, sep=","),
                     src="alphavantage", updated=Sys.time())
       # convert column names to symbol.series
       cn <- colnames(mat)
@@ -1541,9 +1507,6 @@ getSymbols.tiingo <- function(Symbols, env, api.key,
          " cannot be loaded.", call.=FALSE)
   }
   
-  tmp <- tempfile()
-  on.exit(file.remove(tmp))
-  
   downloadOne <- function(sym, default.return.class, default.periodicity) {
     
     return.class <- getSymbolLookup()[[sym]]$return.class
@@ -1575,25 +1538,27 @@ getSymbols.tiingo <- function(Symbols, env, api.key,
                   "?startDate=", from.strftime,
                   "&endDate=", to.strftime,
                   "&format=", data.type,
-                  "&token=", api.key,
                   "&columns=", paste0(return.columns, collapse=","))
     # If rate limit is hit, the csv API returns HTTP 200 (OK), while json API
     # returns HTTP 429. The latter caused download.file() to error, but the
     # contents of 'tmp' still contain the error message.
-    response <- curl::curl_fetch_disk(URL, tmp)
+    h <- curl::new_handle()
+    curl::handle_setheaders(h, Authorization = paste("Token", api.key))
+    response <- curl::curl_fetch_memory(URL, h)
+    response.data <- rawToChar(response$content)
 
     if (data.type == "json") {
-      stock.data <- jsonlite::fromJSON(tmp)
+      stock.data <- jsonlite::fromJSON(response.data)
       if (verbose) cat("done.\n")
     } else {
-      stock.data <- read.csv(tmp, as.is=TRUE)
+      stock.data <- read.csv(text=response.data, as.is=TRUE)
     }
     # check for error
     if (!all(return.columns %in% names(stock.data))) {
       if (data.type == "json") {
-        msg <- jsonlite::fromJSON(tmp)$detail
+        msg <- stock.data$detail
       } else {
-        msg <- readLines(tmp, warn=FALSE)
+        msg <- readLines(response.data, warn=FALSE)
       }
       msg <- sub("Error: ", "", msg)
       stop(msg, call. = FALSE)
